/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.service

import com.sybotan.base.extensions.rootCause
import com.sybotan.base.extensions.toJson
import com.sybotan.database.SAbstractDao
import com.sybotan.database.SFilter
import com.sybotan.database.extensions.keyMap
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.requests.*
import com.sybotan.service.models.responses.*
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.RequestBody
import kotlin.collections.ArrayList

/**
 * 对象访问服务类基类
 *
 * @author  庞利祥 <sybotan@126.com>
 */
open class SObjectService<ENTITY: Any>
    : SBaseService<ENTITY> {
    /** 伴生对象 */
    companion object {
        /** 日志记录器 */
        private val logger = LoggerFactory.getLogger(SObjectService::class.java)
    } // Companion object

    /**
     * 构造函数
     *
     * @param   entityClazz     实体类
     */
    constructor(entityClazz: Class<ENTITY>): super(entityClazz)

    /**
     * 构造函数
     *
     * @param   daoObject       数据访问对象
     */
    constructor(daoObject: SAbstractDao<ENTITY>): super(daoObject)

    /**
     * 创建对象
     *
     * @param   request         请求对象
     * @return  创建结果应答信息
     */
    fun createList(@RequestBody request: SCreateRequest<ENTITY>): SCreateResponse<ENTITY> {
        return try {
            if (request.isMerge) {
                for (entity in request.content) {
                    onCreateBefore(entity)
                    replace(entity)
                }
            } else {
                for (entity in request.content) {
                    onCreateBefore(entity)
                    insert(entity)
                }
            }
            onCreateSuccess(request.content)
            SCreateResponse(request.content)
        } catch (e: Exception) {
            onCreateFailure(request.content)
            e.printStackTrace()
            SCreateResponse(SResponseType.failure, e.message!!)
        }
    } // Fun createList()

    /**
     * 删除对象
     *
     * @param   entityList       对象ID列表
     * @param   withFilterList   公共过滤条件
     * @return  删除结果应答信息
     */
    fun deleteByKeysList(@RequestBody entityList: ArrayList<ENTITY>, withFilterList: ArrayList<SFilter>? = null): SBaseResponse {
        return try {
            for (entity in entityList) {
                val filterList = ArrayList<SFilter>()
                entity.keyMap().forEach {
                    filterList.add(SFilter.eq(it.key, it.value))
                }

                if (withFilterList != null) {
                    filterList.addAll(withFilterList)
                }

                onDeleteBefore(entity)

                if (!delete(filterList)) {
                    onDeleteFailure(entityList)
                    return SBaseResponse(SResponseType.failure, "删除失败！")
                }
            }
            onDeleteSuccess(entityList)
            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Fun deleteByKeysList()

    /**
     * 根据条件删除对象
     *
     * @param   filterList  删除条件
     * @return  删除结果应答信息
     */
    fun batchDelete(@RequestBody filterList: ArrayList<SFilter>): SBaseResponse {
        return if (delete(filterList)) {
            SBaseResponse(SResponseType.success)
        } else {
            SBaseResponse(SResponseType.failure, "删除失败！")
        }
    } // Fun batchDelete()

    /**
     * 更新对象
     *
     * @param   request         更新请求
     * @param   withFilterList  公共过滤条件
     * @return  更新结果应答信息
     */
    fun updateList(@RequestBody request: SUpdateRequest<ENTITY>, withFilterList: ArrayList<SFilter>? = null): SBaseResponse {
        val entityList = request.content
        return try {
            for (entity in entityList!!) {
                val filterList = ArrayList<SFilter>()
                entity.keyMap().forEach {
                    filterList.add(SFilter.eq(it.key, it.value))
                }

                if (withFilterList != null) {
                    filterList.addAll(withFilterList)
                }

                onUpdateBefore(entity)
                if (!update(entity, request.projection, filterList)) {
                    onUpdateFailure(entityList)
                    return SBaseResponse(SResponseType.failure, "更新失败！")
                }
            }

            onUpdateSuccess(entityList)
            SBaseResponse(SResponseType.success)
        } catch (e: Exception){
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Fun update()

//    /**
//     * 批量更新对象
//     *
//     * @param   request         批量更新请求
//     * @param   withFilterList  公共过滤条件
//     * @return  更新结果应答信息
//     */
//    fun batchUpdate(@RequestBody request: SBatchUpdateRequest<ENTITY>, withFilterList: ArrayList<SFilter>? = null): SBaseResponse {
//        val filterList = ArrayList<SFilter>()
//        if (withFilterList != null) {
//            filterList.addAll(withFilterList)
//        }
//
//        if (request.filters != null) {
//            filterList.addAll(request.filters!!)
//        }
//
//        return if (update(request.content!!, request.projection!!, filterList)) {
//            SBaseResponse(SResponseType.success)
//        } else {
//            SBaseResponse(SResponseType.failure, "更新失败！")
//        }
//    } // Fun batchUpdate()

    /**
     * 根据条件分页查询
     *
     * @param   request         查询请求
     * @param   withFilterList  公共过滤条件
     * @return  查询结果应答信息
     */
    fun pageQuery(@RequestBody request: SQueryRequest, withFilterList: ArrayList<SFilter>? = null): SQueryResponse<ENTITY> {
        return try {
            val queryBuilder = select(request.filters)
            if (null != withFilterList) {
                queryBuilder.filterList.addAll(withFilterList)
                logger.debug("filterList = ${queryBuilder.filterList.toJson()}")
            }

            /** 排序条件 */
            if (null != request.orders) {
                queryBuilder.order(request.orders!!)
            }

            /** 去重 */
            if (null != request.distinct && request.distinct!!) {
                queryBuilder.distinct()
            }

            /** 映射标志 */
            if (null != request.projection) {
                queryBuilder.projection(request.projection!!)
            }

            /** 分组统计 */
            if (null != request.group && !request.group!!.nameList.isNullOrEmpty()) {
                // 分组属性名列表
                queryBuilder.group(*request.group!!.nameList.toArray(emptyArray<String>()))
                // 是否统计个数
                queryBuilder.count(request.group!!.count ?: false)
                // 求和
                if (!request.group!!.sumList.isNullOrEmpty()) {
                    queryBuilder.sum(*request.group!!.sumList.toArray(emptyArray<String>()))
                }
                // 求平均值
                if (!request.group!!.avgList.isNullOrEmpty()) {
                    queryBuilder.avg(*request.group!!.avgList.toArray(emptyArray<String>()))
                }
                // 取最大值
                if (!request.group!!.maxList.isNullOrEmpty()) {
                    queryBuilder.max(*request.group!!.maxList.toArray(emptyArray<String>()))
                }
                // 取最小值
                if (!request.group!!.minList.isNullOrEmpty()) {
                    queryBuilder.min(*request.group!!.minList.toArray(emptyArray<String>()))
                }
            }

            onQueryBefore(queryBuilder)
            val pageInfo = queryBuilder.pageQuery(request.pageNumber,request.pageSize)
            /** 如果需要进行级联查询 */
            if (request.cascade != null && request.cascade!!.size > 0 &&
                    pageInfo.content != null && pageInfo.content!!.size > 0) {
                @Suppress("UNCHECKED_CAST")
                processCascadeQuery(pageInfo.content!! as ArrayList<Any>, request.cascade!!)
            }

            onQuerySuccess(pageInfo)
            SQueryResponse(pageInfo)
        } catch (e : Exception){
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.rootCause()?.message ?: e.message ?: "未知异常！")
        }
    } // Fun pageQuery()

    /**
     * 根据条件统计
     *
     * @param   request         查询条件请求
     * @param   withFilterList  公共过滤条件
     * @return  查询结果应答信息
     */
    fun count(@RequestBody request: SCountRequest, withFilterList: ArrayList<SFilter>? = null): SCountResponse {
        return try {
            val queryBuilder = select(request.filters)

            if (null != withFilterList) {
                queryBuilder.filterList.addAll(withFilterList)
            }

            onCountBefore(queryBuilder)
            val count = queryBuilder.execCount()
            onCountSuccess(count)
            SCountResponse(count)
        } catch (e : Exception){
            e.printStackTrace()
            SCountResponse(0, SResponseType.failure, e.rootCause()?.message ?: e.message ?: "未知异常！")
        }
    } // Fun count()
} // Class SObjectService
