/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.service.json

import com.alibaba.fastjson.annotation.JSONField
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector
import com.fasterxml.jackson.databind.PropertyName
import com.fasterxml.jackson.databind.introspect.Annotated
import com.sybotan.base.extensions.toJson

/**
 * swagger使用的json注解拦截器
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SFastjsonAnnotationIntrospector : JacksonAnnotationIntrospector() {

    /**
     * 判断注解是否为JSONField类型
     *
     * @param   ann         注解
     * @return  如果有JSONField注解，则返回true；否则false.
     */
    override fun isAnnotationBundle(ann: Annotation): Boolean {
        println("ann = ${ann.toJson()}")
        return if (ann.annotationClass == JSONField::class.java) {
            println("ann = ======")
            true
        } else {
            println("ann = -------")
            super.isAnnotationBundle(ann)
        }
    } // Fun isAnnotationBundle()

    /**
     * 为序列为操作查找属性名
     *
     * @param   a           注解
     * @return  属性名
     */
    override fun findNameForSerialization(a: Annotated): PropertyName? {
        val nameForSerialization = super.findNameForSerialization(a)
        if (nameForSerialization == null || nameForSerialization === PropertyName.USE_DEFAULT) {
            val jsonField = _findAnnotation(a, JSONField::class.java)
            if (jsonField != null) {
                return PropertyName.construct(jsonField.name)
            }
        }
        return nameForSerialization
    } // Fun findNameForSerialization()

    /**
     * 为反序列为操作查找属性名
     *
     * @param   a           注解
     * @return  属性名
     */
    override fun findNameForDeserialization(a: Annotated): PropertyName? {
        val nameForDeserialization = super.findNameForDeserialization(a)
        if (nameForDeserialization == null || nameForDeserialization === PropertyName.USE_DEFAULT) {
            val jsonField = _findAnnotation(a, JSONField::class.java)
            if (jsonField != null) {
                return PropertyName.construct(jsonField.name)
            }
        }
        return nameForDeserialization
    } // Fun findNameForDeserialization()
} // Class SFastjsonAnnotationIntrospector