/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.service.utils

import org.slf4j.LoggerFactory
import javax.imageio.ImageIO
import javax.servlet.ServletOutputStream
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import java.awt.*
import java.awt.image.BufferedImage
import java.io.IOException
import java.util.Random

/**
 * 验证码生成器
 *
 * @author 庞利祥 <sybotan@126.com>
 */
object SRandCodeUtil {
    // 日志
    private val logger = LoggerFactory.getLogger(SRandCodeUtil::class.java)

    /**
     * 生成验证码
     *
     * @param request  客户端请求对象
     * @param response 服务器应答对象
     */
    fun checkCode(request: HttpServletRequest, response: HttpServletResponse) {
        val width = 195// 验证码图片的宽度
        val height = 26// 验证码图片的高度
        val codeCount = 4// 验证码字符个
        val xx: Int
        val fontHeight: Int// 字体高度
        val codeY: Int
        val codeSequence = charArrayOf('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '1', '2', '3', '4', '5', '6', '7', '8', '9')
        xx = width / (codeCount + 1)
        fontHeight = height - 2
        codeY = height - 5

        // 定义图像buffer
        val buffImg = BufferedImage(width, height, BufferedImage.TYPE_INT_RGB)
        val gd = buffImg.createGraphics()

        // 创建随机数生成器
        val random = Random()

        // 将图像填充为白色
        gd.color = Color.WHITE
        gd.fillRect(0, 0, width, height)

        // 创建字体，字体的大小应该根据图片的高度来定
        // 字体
        val fontTypes = arrayOf("tahoma", "Atlantic Inline", "fantasy", "Times New Roman", "Georgia", "Arial", "Helvetica", "sans-serif", "System")
        val fontTypesLength = fontTypes.size
        val font = Font(fontTypes[random.nextInt(fontTypesLength)], Font.BOLD, fontHeight)
        // 设置字体
        gd.font = font

        gd.color = Color.red
        for (i in 0..99) {
            val x = random.nextInt(width)
            val y = random.nextInt(height)
            gd.drawOval(x, y, 0, 0)
        }

        // randomCode用于保存随机产生的验证码，以便用户登录后进行验证
        val randomCode = StringBuffer()
        var red: Int
        var green: Int
        var blue: Int

        // 随机产生codeCount数字的验证码
        for (i in 0 until codeCount) {
            // 得到随机产生的验证码数字
            val strRand = codeSequence[random.nextInt(34)].toString()
            // 产生随机的颜色分量来构建颜色值，这样输出的每位数字的颜色值都将不同
            red = random.nextInt(200)
            green = random.nextInt(200)
            blue = random.nextInt(200)

            // 用随机产生的颜色将验证码绘制到图像中
            gd.color = Color(red, green, blue)
            gd.drawString(strRand, (i + 1) * xx, codeY)

            // 将产生的四个随机数组合在
            randomCode.append(strRand)
        }

        // 将四位数字的验证码保存到Session中
        logger.debug("verifyCode = $randomCode")
        val session = request.session
        session.setAttribute("verifyCode", randomCode.toString())

        // 禁止图像缓存
        response.setHeader("Pragma", "no-cache")
        response.setHeader("Cache-Control", "no-cache")
        response.setDateHeader("Expires", 0)
        response.contentType = "image/jpeg"

        // 将图像输出到Servlet输出流中
        val sos: ServletOutputStream
        try {
            sos = response.outputStream
            ImageIO.write(buffImg, "jpeg", sos)
            sos.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return
    } // Fun checkCode()

} // Class RandCOdeUtil
