/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.sso.controllers

import com.sybotan.base.extensions.toJson
import com.sybotan.service.SObjectService
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SQueryRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCreateResponse
import com.sybotan.service.models.responses.SQueryResponse
import com.sybotan.sso.models.entities.SSsoUser
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody

/**
 * 单点登录用户接口
 *
 * @author 庞利祥 <sybotan@126.com>
 */
open class SSsoUserController<USER: SSsoUser> {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(SSsoUserController::class.java)
    } // Companion object

    protected lateinit var userService: SObjectService<USER> // = SDataService(SMybatisDao(SSsoUser::class.java))

    /**
     * 创建用户
     */
    @ApiOperation(value = "创建单点登录用户", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<USER>): SCreateResponse<USER> {
        logger.debug("user =${request.toJson()}")
        return userService.createList(request)
    } // Fun create()

    /**
     * 更新用户
     */
    @ApiOperation(value = "更新单点登录用户", notes = "")
    @PostMapping(value = ["/update"])
    fun updateList(@RequestBody request: SUpdateRequest<USER>): SBaseResponse {
        return userService.updateList(request)
    } // Fun update()

    /**
     * 删除用户
     */
    @ApiOperation(value = "删除单点登录用户", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody entityList: ArrayList<USER>): SBaseResponse {
        return userService.deleteByKeysList(entityList)
    } // Fun delete()

    /**
     * 查询单点登录用户
     *
     * @param   request     查询单点登录用户
     * @return  查询结果应答信息
     */
    @ApiOperation(value="查询单点登录用户", notes="")
    @PostMapping(value = ["/query"])
    fun queryPage(@RequestBody request: SQueryRequest): SQueryResponse<USER> {
        return userService.pageQuery(request)
    } // Fun queryPage()
} // Class SSsoUserController
