/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.service.models.requests

import com.sybotan.database.SGroup
import com.sybotan.service.models.SCascadeQuery
import io.swagger.annotations.ApiModel
import io.swagger.annotations.ApiModelProperty

/**
 * 查询请求对象基类
 *
 * @author  庞利祥 <sybotan@126.com>
 */
@ApiModel(description = "查询请求对象基类")
open class SQueryRequest {
    companion object {
        /** 设置接口一次可取得的最大记录数 */
        var maxRow = 1000
    } // Companion object

    /** 第几页 */
    @ApiModelProperty(value = "第几页(从1开始)", example = "1")
    var pageNumber: Int = 1
        set(value) {            // 限定pageNumber不能小于1
            field = value
            if (value < 1) {
                field = 1
            }
        }

    /** 每页记录数 */
    @ApiModelProperty(value = "每页记录数(0 < 记录数 < 1000)", example = "50")
    var pageSize: Int = 50
        set(value) {            // 限定pageSize不能小于1且不能大于maxRow
            field = value
            if (value < 1 ) {
                field = 1
            }
            if (value > maxRow) {
                field = maxRow
            }
            // maxRow一次设置有效，之后将自动设备为1000
            maxRow = 1000
        }

    /** 查询条件 */
    @ApiModelProperty(value = "查询条件")
    var filters: String? = null

    /** 排序条件 */
    @ApiModelProperty(value = "排序条件", example = "name asc, createTime desc")
    var orders: String? = null

    /** 去除重复记录 */
    @ApiModelProperty(value = "去除重复记录")
    var distinct: Boolean? = null

    /** 字段影射，空为查询所有字段 */
    @ApiModelProperty(value = "字段影射，空为查询所有字段")
    var projection: ArrayList<String>? = null

    /** 级联对象列表 */
    @ApiModelProperty(value = "级联对象列表")
    var cascade: ArrayList<SCascadeQuery>? = null

    /** 分组统计 */
    @ApiModelProperty(value = "分组")
    var group: SGroup? = null
} // Class SQueryRequest